/***************************************************************************
 * Plug-in for mt9v111 image sensor connected to a m5602                   *
 *                                                                         *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               *
 ***************************************************************************/

#include "m560x_sensor.h"

static struct m560x_sensor mt9v111;
static u8 i2c_data[5+1];

static int mt9v111_init(struct m560x_camera* cam)
{
  return 0;
}

static int mt9v111_get_ctrl(struct m560x_camera* cam, struct v4l2_control * ctrl)
{
  return 0;
}

static int mt9v111_set_ctrl(struct m560x_camera* cam, const struct v4l2_control * ctrl)
{
  return 0;
}

static int mt9v111_set_crop(struct m560x_camera* cam, const struct v4l2_rect * rect)
{
  return 0;
}

static int mt9v111_set_pix_format(struct m560x_camera* cam, const struct v4l2_pix_format * pix)
{
  return 0;
}

static int mt9v111_i2c_write(struct m560x_camera* cam, const unsigned char register_id, const unsigned char * i2c_data)
{
  return 0;
}

static int mt9v111_i2c_read(struct m560x_camera* cam, const unsigned char register_id, unsigned char * data)
{
  int result = 0;

  data[0] = 2;
  
  do {
    result = read_m5602_register(cam,0x816d);
  } while ( (result & 0x80) == 0x80);
  write_m5602_register(cam,0x816a,mt9v111.i2c_slave_id);
  write_m5602_register(cam,0x816b,register_id);
  write_m5602_register(cam,0x8168,0x1a);
  data[1] = read_m5602_register(cam,0x816c);
  
  do {
    result = read_m5602_register(cam,0x816d);
  } while ( (result & 0x80) == 0x80);
  write_m5602_register(cam,0x816a,mt9v111.i2c_slave_id);
  write_m5602_register(cam,0x816b,0x80);
  write_m5602_register(cam,0x8168,0x1a);
  data[2] = read_m5602_register(cam,0x816c);

  return 0;
}

static struct m560x_sensor mt9v111 = {
  .name = "MT9V111",
  .i2c_slave_id = 0xBA,
  .init = &mt9v111_init,
  .qctrl = {
    {
      .id = V4L2_CID_EXPOSURE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "exposure",
      .minimum = 0x00,
      .maximum = 0x0f,
      .step = 0x01,
      .default_value = 0x06,
      .flags = 0,
    },
    {
      .id = V4L2_CID_GAIN,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "global gain",
      .minimum = 0x00,
      .maximum = (0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0),/*0x6d*/
      .step = 0x01,
      .default_value = 0x00,
      .flags = 0,
    },
    {
      .id = V4L2_CID_HFLIP,
      .type = V4L2_CTRL_TYPE_BOOLEAN,
      .name = "horizontal mirror",
      .minimum = 0,
      .maximum = 1,
      .step = 1,
      .default_value = 0,
      .flags = 0,
    },
    {
      .id = V4L2_CID_VFLIP,
      .type = V4L2_CTRL_TYPE_BOOLEAN,
      .name = "vertical mirror",
      .minimum = 0,
      .maximum = 1,
      .step = 1,
      .default_value = 0,
      .flags = 0,
    },
    {
      .id = V4L2_CID_RED_BALANCE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "red balance",
      .minimum = 0x00,
      .maximum = (0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0),
      .step = 0x01,
      .default_value = 0x00,
      .flags = 0,
    },
    {
      .id = V4L2_CID_BLUE_BALANCE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "blue balance",
      .minimum = 0x00,
      .maximum = (0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0),
      .step = 0x01,
      .default_value = 0x00,
      .flags = 0,
    },
    {
      .id = M560X_V4L2_CID_GREEN_BALANCE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "green balance",
      .minimum = 0x00,
      .maximum = ((0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0)),
      .step = 0x01,
      .default_value = 0x00,
      .flags = 0,
    },
  },
  .get_ctrl = &mt9v111_get_ctrl,
  .set_ctrl = &mt9v111_set_ctrl,
  .cropcap = {
    .bounds = {
      .left = 0,
      .top = 0,
      .width = 640,
      .height = 480,
    },
    .defrect = {
      .left = 0,
      .top = 0,
      .width = 640,
      .height = 480,
    },
  },
  .set_crop = &mt9v111_set_crop,
  .pix_format = {
    .width = 640,
    .height = 480,
    .pixelformat = V4L2_PIX_FMT_SBGGR8,
    .priv = 8,
  },
  .set_pix_format = &mt9v111_set_pix_format,
  .i2c_write = &mt9v111_i2c_write,
  .i2c_read = &mt9v111_i2c_read,
};

int m5602_probe_mt9v111(struct m560x_camera* cam)
{
  int result = 0;
  
  result = mt9v111_i2c_read(cam,0x00,i2c_data);
  info("[%s:%d] " "probe mt9v111 sensor id %02x %02x", __FUNCTION__, __LINE__ ,i2c_data[1], i2c_data[2]);
  
  if ( (i2c_data[1] == 0x14) && (i2c_data[2] == 0x59)) {
    info("[%s:%d] " "detected mt9v111 sensor", __FUNCTION__, __LINE__ );
    m560x_attach_sensor(cam, &mt9v111);
  }
  else
    return -ENODEV;
  
  return 0;
}
