/***************************************************************************
 * Plug-in for mt9v011 image sensor connected to a m5603                   *
 *                                                                         *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               *
 ***************************************************************************/


#include "m560x_sensor.h"
#include "m560x.h"

static struct m560x_sensor mt9v011;
static u8 i2c_data[4];

static int mt9v011_init(struct m560x_camera* cam)
{
//  m5603_init_from_script(cam,"AP_ON");
//   m5603_init_from_script(cam,"R_VGA");
  m5603_init_from_script(cam,"EXTRA3");
  return 0;
}

static int mt9v011_get_ctrl(struct m560x_camera* cam, struct v4l2_control * ctrl)
{
  unsigned char id1,id2;
  
   switch (ctrl->id) {
     case V4L2_CID_EXPOSURE:
       read_sensor_register(cam,0x09BA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       ctrl->value = (int) ((unsigned int) (id1 << 8) & (unsigned int) (id2));
       return 0;
     case V4L2_CID_GAIN:
       read_sensor_register(cam,0x35BA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       break;
     case V4L2_CID_RED_BALANCE:
       read_sensor_register(cam,0x2DBA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       break;
     case V4L2_CID_BLUE_BALANCE:
       read_sensor_register(cam,0x2CBA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       break;
     case M560X_V4L2_CID_GREEN_BALANCE:
       read_sensor_register(cam,0x35BA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       break;
     case V4L2_CID_VFLIP:
       read_sensor_register(cam,0x20BA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       ctrl->value = id1 & 0x80 ? 1 : 0;
       return 0;
     case V4L2_CID_HFLIP:
       read_sensor_register(cam,0x20BA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       ctrl->value = id1 & 0x40 ? 1 : 0;
      return 0;
     case M560X_V4L2_CID_EXTRA_GAIN:
       read_sensor_register(cam,0x35BA,i2c_data);
       id1 = i2c_data[1];
       read_sensor_register(cam,0x80BA,i2c_data);
       id2 = i2c_data[1];
       ctrl->value = id1 & 0x0c ? 1 : 0;
       return 0;
     default:
      return 0;
  }
  switch (ctrl->id) {
    case V4L2_CID_GAIN:
    case V4L2_CID_RED_BALANCE:
    case V4L2_CID_BLUE_BALANCE:
    case M560X_V4L2_CID_GREEN_BALANCE:
      ctrl->value = (int) ((unsigned int) id2 & 0x7f);
      if ( id1 & 0x01) {
        if ( id2 & 0x80) {
          ctrl->value = ctrl->value << 2;
        }
        else {
          ctrl->value = ctrl->value << 1;
        }
      }
    default:
      return 0;
    
  }
  return 0;
}

static int mt9v011_set_ctrl(struct m560x_camera* cam, const struct v4l2_control * ctrl)
{
//   int err = 0;
  unsigned char id1 = 0,id2 = 0;
  i2c_data[0] = 2;
  
  switch (ctrl->id) {
    case V4L2_CID_GAIN:
    case V4L2_CID_RED_BALANCE:
    case V4L2_CID_BLUE_BALANCE:
    case M560X_V4L2_CID_GREEN_BALANCE:
      if (ctrl->value >= 0 && ctrl->value < 0x40) {
        id1 = 0;
        id2 = (unsigned char) (ctrl->value & 0x7f);
      }
      else if (ctrl->value >= 0x40 && ctrl->value < 0x100) {
        id1 = 0;
        id2 = (unsigned char) ((ctrl->value >> 1) & 0x7f);
        id2 |= 0x80;
      }
      else if (ctrl->value >= 0x100 && ctrl->value < 0x1ff) {
        id1 = 0x01;
        id2 = (unsigned char) ((ctrl->value >> 2) & 0x7f);
        id2 |= 0x80;
      }
      i2c_data[0] = 2;
      i2c_data[1] = id1;
      i2c_data[2] = id2;
      break;
    default:
      break;
  }
  
  switch (ctrl->id) {
    case V4L2_CID_EXPOSURE:
      i2c_data[1] = (unsigned char) ((ctrl->value & 0xf00) >> 8);
      i2c_data[2] = (unsigned char) (ctrl->value & 0xff);
      write_sensor_register(cam, 0x09BA, i2c_data);
      break;  
    case V4L2_CID_GAIN: 
      write_sensor_register(cam, 0x35BA, i2c_data);
      break;
    case V4L2_CID_RED_BALANCE:
      write_sensor_register(cam, 0x2DBA, i2c_data);
      break;
    case V4L2_CID_BLUE_BALANCE:
      write_sensor_register(cam, 0x2CBA, i2c_data);
      break;
    case M560X_V4L2_CID_GREEN_BALANCE:
      write_sensor_register(cam, 0x2BBA, i2c_data);
      write_sensor_register(cam, 0x2EBA, i2c_data);
      break;
    case V4L2_CID_VFLIP:
      read_sensor_register(cam,0x20BA, i2c_data);
      id1 = i2c_data[1];
      read_sensor_register(cam,0x80BA, i2c_data);
      id2 = i2c_data[1];
      i2c_data[0] = 2;
      i2c_data[1] = ctrl->value ? (id1 | 0x40) : (id1 & 0xbf);
      i2c_data[2] = ctrl->value ? (id2 | 0x00) : (id2 & 0xff);
      write_sensor_register(cam, 0x20BA, i2c_data);
      break;
    case V4L2_CID_HFLIP:
      read_sensor_register(cam,0x20BA, i2c_data);
      id1 = i2c_data[1];
      read_sensor_register(cam,0x80BA, i2c_data);
      id2 = i2c_data[1];
      i2c_data[0] = 2;
      i2c_data[1] = ctrl->value ? (id1 | 0x80) : (id1 & 0x7f);
      i2c_data[2] = id2;
      write_sensor_register(cam, 0x20BA, i2c_data);
      break;
    case M560X_V4L2_CID_EXTRA_GAIN:
      read_sensor_register(cam,0x35BA, i2c_data);
      id1 = i2c_data[1];
      read_sensor_register(cam,0x80BA, i2c_data);
      id2 = i2c_data[1];
      i2c_data[0] = 2;
      i2c_data[1] = ctrl->value ? (i2c_data[1] | 0x0c) : (i2c_data[1] & 0xf3);
      i2c_data[2] = id2;
      write_sensor_register(cam, 0x35BA, i2c_data);
      break;
     
    
    default:
      break;
    
  }
  
  return 0;
}

static int mt9v011_set_crop(struct m560x_camera* cam, const struct v4l2_rect * rect)
{
  return 0;
}

static int mt9v011_set_pix_format(struct m560x_camera* cam, const struct v4l2_pix_format * pix)
{
  return 0;
}


static int mt9v011_i2c_write(struct m560x_camera* cam, const unsigned char register_id, const unsigned char * i2c_data)
{
  return 0;
}


static int mt9v011_i2c_read(struct m560x_camera* cam, const unsigned char register_id, unsigned char * i2c_data)
{
  struct usb_device* udev = cam->udev;
  int result = 0;
  unsigned short ali_register;
  unsigned char value;
  
  i2c_data[0] = 2;
  ali_register = ((unsigned short) register_id << 8) | (unsigned short) mt9v011.i2c_slave_id;
  result = usb_control_msg(udev, usb_rcvctrlpipe(udev, 0), 0x04, 0xc0,0x0018,ali_register,&value,1,100);
  i2c_data[1] = value;
  ali_register = ((unsigned short) 0x80 << 8) | (unsigned short) mt9v011.i2c_slave_id;
  result = usb_control_msg(udev, usb_rcvctrlpipe(udev, 0), 0x04, 0xc0,0x0018,ali_register,&value,1,100);
  i2c_data[2] = value;
  return 0;
}

static struct m560x_sensor mt9v011 = {
  .name = "MT9V011",
  .i2c_slave_id = 0xBA,
  .init = &mt9v011_init,
  .qctrl = {
    {
      .id = V4L2_CID_EXPOSURE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "exposure",
      .minimum = 0x0000,
      .maximum = 0x03ff,
      .step = 0x01,
      .default_value = 0x01fc,
      .flags = 0,
    },
    {
      .id = V4L2_CID_GAIN,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "global gain",
      .minimum = 0x00,
      .maximum = 0x1fc,
      .step = 0x01,
      .default_value = 0x7f,
      .flags = 0,
    },
    {
      .id = V4L2_CID_HFLIP,
      .type = V4L2_CTRL_TYPE_BOOLEAN,
      .name = "horizontal mirror",
      .minimum = 0,
      .maximum = 1,
      .step = 1,
      .default_value = 0,
      .flags = 0,
    },
    {
      .id = M560X_V4L2_CID_EXTRA_GAIN,
      .type = V4L2_CTRL_TYPE_BOOLEAN,
      .name = "extra gain",
      .minimum = 0,
      .maximum = 1,
      .step = 1,
      .default_value = 0,
      .flags = 0,
    },
    {
      .id = V4L2_CID_VFLIP,
      .type = V4L2_CTRL_TYPE_BOOLEAN,
      .name = "vertical mirror",
      .minimum = 0,
      .maximum = 1,
      .step = 1,
      .default_value = 0,
      .flags = 0,
    },
    {
      .id = V4L2_CID_RED_BALANCE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "red balance",
      .minimum = 0x00,
//       .maximum = (0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0),
      .maximum = 0x1ff,
      .step = 0x01,
      .default_value = 0x7f,
      .flags = 0,
    },
    {
      .id = V4L2_CID_BLUE_BALANCE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "blue balance",
      .minimum = 0x00,
      .maximum = 0x00,
//       .maximum = (0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0),
      .maximum = 0x1ff,
      .step = 0x01,
      .default_value = 0x7f,
      .flags = 0,
    },
    {
      .id = M560X_V4L2_CID_GREEN_BALANCE,
      .type = V4L2_CTRL_TYPE_INTEGER,
      .name = "green balance",
      .minimum = 0x00,
//       .maximum = ((0x3f-0x10)+(0x7f-0x60)+(0xff-0xe0)),
      .maximum = 0x1ff,
      .step = 0x01,
      .default_value = 0x7f,
      .flags = 0,
    },
  },
  .get_ctrl = &mt9v011_get_ctrl,
  .set_ctrl = &mt9v011_set_ctrl,
  .cropcap = {
    .bounds = {
      .left = 0,
      .top = 0,
      .width = 640,
      .height = 480,
    },
    .defrect = {
      .left = 0,
      .top = 0,
      .width = 640,
      .height = 480,
    },
  },
  .set_crop = &mt9v011_set_crop,
  .pix_format = {
    .width = 640,
    .height = 480,
    .pixelformat = V4L2_PIX_FMT_RGB24,
    .priv = 8,
  },
  .set_pix_format = &mt9v011_set_pix_format,
  .i2c_write = &mt9v011_i2c_write,
  .i2c_read = &mt9v011_i2c_read,
};


int m5603_probe_mt9v011(struct m560x_camera* cam)
{
  int result = 0;
  
  result = mt9v011_i2c_read(cam,0x00,i2c_data);
  info("[%s:%d] " "probe mt9v011 sensor id %02x %02x", __FUNCTION__, __LINE__ ,i2c_data[1], i2c_data[2]);
  if ( (i2c_data[1] == 0x82) && (i2c_data[2] == 0x32)) {
    info("[%s:%d] " "detected mt9v011 sensor", __FUNCTION__, __LINE__ );
    m560x_attach_sensor(cam, &mt9v011);
  }
  else
    return -ENODEV;
  
  return 0;
}
